@@ -703,6 +703,53 @@ class PostgresqlBackend(DatabaseBackend):
         return super(PostgresqlBackend, self).list_tables(schema=current_schema)
 
 
+class RedshiftBackend(PostgresqlBackend):
+    def list_tables(self):
+        current_schema = self.execute("SELECT current_schema()").fetchone()[0]
+        return super(PostgresqlBackend, self).list_tables(schema=current_schema)
+
+    # Redshift does not support ROLLBACK TO SAVEPOINT
+    def savepoint(self, id):
+        pass
+
+    def savepoint_release(self, id):
+        pass
+
+    def savepoint_rollback(self, id):
+        self.rollback()
+
+    # Redshift does not enforce primary and unique keys
+    def _insert_lock_row(self, pid, timeout, poll_interval=0.5):
+        poll_interval = min(poll_interval, timeout)
+        started = time.time()
+        while True:
+            with self.transaction():
+                # prevents isolation violation errors
+                self.execute("LOCK {}".format(self.lock_table_quoted))
+                cursor = self.execute(
+                    "SELECT pid FROM {}".format(self.lock_table_quoted)
+                )
+                row = cursor.fetchone()
+                if not row:
+                    self.execute(
+                        "INSERT INTO {} (locked, ctime, pid) "
+                        "VALUES (1, :when, :pid)".format(
+                            self.lock_table_quoted
+                        ),
+                        {"when": datetime.utcnow(), "pid": pid},
+                    )
+                    return
+                elif timeout and time.time() > started + timeout:
+                    raise exceptions.LockTimeout(
+                        "Process {} has locked this database "
+                        "(run yoyo break-lock to remove this lock)".format(
+                            row[0]
+                        )
+                    )
+                else:
+                    time.sleep(poll_interval)
+
+
 class SnowflakeBackend(DatabaseBackend):
 
     driver_module = "snowflake.connector"
